/*
  Wifimanager_config_ESP8266_Siton_to_EmonCMS.ino
  Program prijima data z menice Siton210 ve formatu EasyTransfer a po zpracovani je odesila pres wifi na server Emoncms.
  Pro nastaveni Wifi a serveru Emoncms pouziva WiFiManager s ulozenim nastaveni do config souboru.
  Spusteni konfigurace tlacitkem Flash na modulu NodeMCU.
  Knihovnu WiFiManager(Ken Taylor) importovat rucne ze stazeneho zip. souboru. Pozor na konflikt knihoven WiFiManager.
  HW: NodeMCU s ESP8266 

  upraveno z example ConfigOnSfitchFS.ino, vcetne ArduinoJson v6.
  Arduino IDE: 1.8.5; 
  Tomas Nevrela; tnweb.tode.cz
  6.4.2020
  
D1 GPIO5
D2 GPIO4
D3 GPIO0
D4 GPIO2
D5 GPIO14

*/

#include <FS.h>
#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino
#include <ArduinoJson.h>          //https://github.com/bblanchon/ArduinoJson
#include <SoftwareSerial.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <WiFiManager.h>          //https://github.com/kentaylor/WiFiManager



const int PIN_LED = 2; // D4 ,LOW -> LED Sviti.
const int TRIGGER_PIN = 0; // D3 LOW -> spusteni configurace WiFi
const int RESET_PIN = 14; // D5 LOW -> format SPIFFS, reset WiFi setting

const char* CONFIG_FILE = "/config.json";


// Indicates whether ESP has WiFi credentials saved from previous session
bool initialConfig = false;

// Default configuration values
char emoncmsAddress[40] = "emoncms.org";// 192.168.xx.xx
word port = 80;
char emoncmsApiKey[35] = "xxxxxxxxxx";


SoftwareSerial mySerial(5, 4, false, 64); //D1, D2(rxPin, txPin, inverse_logic, buffer size)
byte nodeID;
int napeti;
int proud;
int vykon;
int teplota;
unsigned long vyroba;
byte byteRec;
byte delka;
byte cs;
byte prijem[40];
int perKomunikace = 10; // perioda odesilani sec.
unsigned long currentMillis = 0;
unsigned long lastMillis;
bool dataOK = false;
// Function Prototypes

bool readConfigFile();
bool writeConfigFile();

//=========================================== Setup ===============================
void setup() {
  // Put your setup code here, to run once
  Serial.begin(115200);
  mySerial.begin(9600); //serial port pro data ze Sitonu
  Serial.println("\n Starting");

  // Initialize the LED digital pin as an output.
  pinMode(PIN_LED, OUTPUT);
  // Initialize trigger pin
  pinMode(TRIGGER_PIN, INPUT_PULLUP);
  pinMode(RESET_PIN, INPUT_PULLUP);


  // Mount the filesystem
  bool result = SPIFFS.begin();
  Serial.println("SPIFFS opened: " + result);

  if (!readConfigFile()) {
    Serial.println("Failed to read configuration file, using default values");
  }

  WiFi.printDiag(Serial); //Remove this line if you do not want to see WiFi password printed

  if (WiFi.SSID() == "") {
    Serial.println("We haven't got any access point credentials, so get them now");
    initialConfig = true;
  } else {
    digitalWrite(PIN_LED, HIGH); // Turn LED off as we are not in configuration mode.
    WiFi.mode(WIFI_STA); // Force to station mode because if device was switched off while in access point mode it will start up next time in access point mode.
    unsigned long startedAt = millis();
    Serial.print("After waiting ");
    int connRes = WiFi.waitForConnectResult();
    float waited = (millis() - startedAt);
    Serial.print(waited / 1000);
    Serial.print(" secs in setup() connection result is ");
    Serial.println(connRes);
  }

  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("Failed to connect, finishing setup anyway");
  } else {
    Serial.print("Local ip: ");
    Serial.println(WiFi.localIP());
  }
}

//============================================ Loop ================================
void loop() {
  // is configuration portal requested?
  if ((digitalRead(TRIGGER_PIN) == LOW) || (initialConfig)) {
    Serial.println("Configuration portal requested");
    digitalWrite(PIN_LED, LOW); // turn the LED on by making the voltage LOW to tell us we are in configuration mode.
    //Local intialization. Once its business is done, there is no need to keep it around

    // Extra parameters to be configured
    // After connecting, parameter.getValue() will get you the configured value
    // Format: <ID> <Placeholder text> <default value> <length> <custom HTML> <label placement>
    WiFiManagerParameter p_emoncmsAddress("emoncmsaddress", "EmonCMS Adresa", emoncmsAddress, 40);

    char convertedValue[4];
    sprintf(convertedValue, "%d", port);
    WiFiManagerParameter p_port("port", "Port", convertedValue, 4);
    // Thingspeak API Key - this is a straight forward string parameter
    WiFiManagerParameter p_emoncmsApiKey("emoncmsapikey", "Emoncms-API Key", emoncmsApiKey, 33);

    // Just a quick hint
    WiFiManagerParameter p_hint("<small>*Hint: if you want to reuse the currently active WiFi credentials, leave SSID and Password fields empty</small>");

    // Initialize WiFIManager
    WiFiManager wifiManager;

    if ((digitalRead(RESET_PIN) == LOW)){  // reset nastaveni a WiFi
	SPIFFS.format(); //
	wifiManager.resetSettings();
    }
	
	//add all parameters here

    wifiManager.addParameter(&p_hint);
    wifiManager.addParameter(&p_emoncmsAddress);
    wifiManager.addParameter(&p_port);
    wifiManager.addParameter(&p_emoncmsApiKey);


    // Sets timeout in seconds until configuration portal gets turned off.
    // If not specified device will remain in configuration mode until
    // switched off via webserver or device is restarted.
    // wifiManager.setConfigPortalTimeout(600);

    // It starts an access point
    // and goes into a blocking loop awaiting configuration.
    // Once the user leaves the portal with the exit button
    // processing will continue
    if (!wifiManager.startConfigPortal("ESP_Emoncms")) {
      Serial.println("Not connected to WiFi but continuing anyway.");
    } else {
      // If you get here you have connected to the WiFi
      Serial.println("Connected...yeey :)");
    }


    // Getting posted form values and overriding local variables parameters
    // Config file is written regardless the connection state
    strcpy(emoncmsAddress, p_emoncmsAddress.getValue());
    port = atoi(p_port.getValue());
    strcpy(emoncmsApiKey, p_emoncmsApiKey.getValue());

    writeConfigFile();


    digitalWrite(PIN_LED, HIGH); // Turn LED off as we are not in configuration mode.

    ESP.reset(); // This is a bit crude. For some unknown reason webserver can only be started once per boot up
    // so resetting the device allows to go back into config mode again when it reboots.
    delay(5000);

  }
  //===========================================================================
  //cte ze serial portu data ve formatu EasyTransfer
  if (mySerial.available() >= 0x25)
  {
    delay(10);
    byteRec = mySerial.read();    // Precte byte
    if (byteRec == 0x06) {
      byteRec = mySerial.read();
      if (byteRec == 0x85) {
        delka = mySerial.read();
        cs = delka;
        for (byte i = 0; i < delka ; i++) {
          byteRec = mySerial.read();
          prijem[i] = byteRec;
          cs ^= byteRec; //kontrolni soucet
        }
        //Serial.println();
        byteRec = mySerial.read();
        //Serial.println(byteRec, HEX);
        //Serial.print(cs, HEX);
        if (cs == byteRec) { //kontrolni soucet OK data jsou platna
          dataOK = true;
          nodeID = prijem[0];
          napeti = word(prijem[5], prijem[4]);
          proud  = word(prijem[7], prijem[6]);
          vykon  = word(prijem[9], prijem[8]);
          teplota = word(prijem[11], prijem[10]);

          vyroba  = (long)prijem[23] << 24;
          vyroba += (long)prijem[22] << 16;
          vyroba += (long)prijem[21] << 8;
          vyroba += (long)prijem[20];


          Serial.print("Siton adresa:");
          Serial.print(nodeID);
          Serial.print(" Napeti:");
          Serial.print(napeti);
          Serial.print("V ");
          Serial.print("Proud:");
          Serial.print(proud / 100);
          Serial.print(".");
          Serial.print((proud % 100) / 10);
          Serial.print("A ");
          Serial.print("Vykon:");
          Serial.print(vykon);
          Serial.print("W ");
          Serial.print("Teplota:");
          Serial.print(teplota);
          Serial.print("'C ");
          Serial.print("Vyroba:");
          Serial.print(vyroba);
          Serial.println("Wh");
        }
      }
    }
  }
  else dataOK = false;
  //===========================================================================
  // Odeslani dat na server Emoncms //
  currentMillis = millis();
  if ((unsigned long)(currentMillis - lastMillis >= (perKomunikace * 1000)) && dataOK)
  {
    Serial.print("Connecting to ");
    Serial.println(emoncmsAddress);
    digitalWrite(PIN_LED, LOW);
    // Use WiFiClient class to create TCP connections
    WiFiClient client;

    if (!client.connect(emoncmsAddress, port)) {
      Serial.println("connection failed");
      return;
    }

    //
    // prevod RSSI na kvalitu signalu 0-100%
    long rssi = WiFi.RSSI();
    int quality = 0;
    if (rssi <= -100) {
      quality = 0;
    } else if (rssi >= -50) {
      quality = 100;
    } else {
      quality = 2 * (rssi + 100);
    }
    //
    //Build the JSON for emoncms to send data
    String json = "{Wifi:";
    json += quality;
    json += ",Napeti:";
    json += napeti;
    json += ",Proud:";
    json += proud;
    json += ",Vykon:";
    json += vykon;
    json += ",Teplota:";
    json += teplota;
    json += ",Vyroba:";
    json += vyroba;
    json += "}";

    //Build emoncms URL for sending data to
    String url = "/emoncms/input/post.json?node=";
    url += "Siton_"; //Siton_;
    url += nodeID;
    url += "&json=";
    url += json;
    url += "&apikey=";
    url += emoncmsApiKey;
    Serial.print(String("GET ") + url + " HTTP/1.1\r\n" +
                 "Host: " + emoncmsAddress + "\r\n" +
                 "Connection: close\r\n\r\n");

    client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                 "Host: " + emoncmsAddress + "\r\n" +
                 "Connection: close\r\n\r\n");
    delay(10);
    digitalWrite(PIN_LED, HIGH);

    ///////////////// timeout //////////////////////
    unsigned long timeout = millis();
    while (client.available() == 0) {
      if (millis() - timeout > 5000) {
        Serial.println(">>> Client Timeout !");
        client.stop();
        return;
      }
    }

    // Read all the lines of the reply from server and print them to Serial
    while (client.available()) {

      String line = client.readStringUntil('\r');
      Serial.print(line);

    }

    Serial.println();
    Serial.println("closing connection");

    lastMillis = currentMillis;
  }

}
//=============================================================================
bool readConfigFile() {
  // this opens the config file in read-mode
  File f = SPIFFS.open(CONFIG_FILE, "r");

  if (!f) {
    Serial.println("Configuration file not found");
    return false;
  } else {
    // we could open the file
    size_t size = f.size();
    // Allocate a buffer to store contents of the file.
    std::unique_ptr<char[]> buf(new char[size]);

    // Read and store file contents in buf
    f.readBytes(buf.get(), size);
    // Closing file
    f.close();
    // Using dynamic JSON buffer which is not the recommended memory model, but anyway
    // See https://github.com/bblanchon/ArduinoJson/wiki/Memory%20model
    DynamicJsonDocument json(1024); //upraveno na json v6
    // Parse JSON string
    auto deserializeError = deserializeJson(json, buf.get());
    serializeJson(json, Serial);
    // Test if parsing succeeds.
    if (deserializeError) {
      Serial.println("JSON deserialize failed");
      return false;
    }
    //json.printTo(Serial);

    // Parse all config file parameters, override
    // local config variables with parsed values
    if (json.containsKey("emoncmsAddress")) {
      strcpy(emoncmsAddress, json["emoncmsAddress"]);
    }
    if (json.containsKey("emoncmsApiKey")) {
      strcpy(emoncmsApiKey, json["emoncmsApiKey"]);
    }

    if (json.containsKey("port")) {
      port = json["port"];
    }

  }
  Serial.println("\nConfig file was successfully parsed");
  return true;
}
//=============================================================================
bool writeConfigFile() {
  Serial.println("Saving config file");
  DynamicJsonDocument json(1024);//upraveno na json v6
  

  // JSONify local configuration parameters
  json["emoncmsAddress"] = emoncmsAddress;
  json["port"] = port;
  json["emoncmsApiKey"] = emoncmsApiKey;
  
  // Open file for writing
  File f = SPIFFS.open(CONFIG_FILE, "w");
  if (!f) {
    Serial.println("Failed to open config file for writing");
    return false;
  }

  serializeJson(json, Serial);
  serializeJson(json, f);
  f.close();

  Serial.println("\nConfig file was successfully saved");
  return true;
}

